Package: sm
Type: Package
Title: Smoothing Methods for Nonparametric Regression and Density
        Estimation
Version: 2.2-6.0
Date: 2024-02-17
Author: Adrian Bowman and Adelchi Azzalini. 
         Ported to R by B. D. Ripley <ripley@stats.ox.ac.uk> up to version 2.0,
	      version 2.1 by Adrian Bowman and Adelchi Azzalini,
	      version 2.2 by Adrian Bowman.	 	 
Maintainer: Adrian Bowman <adrian.bowman@glasgow.ac.uk>
Depends: R (>= 3.1.0)
Suggests: rgl, misc3d, interp, gam, tkrplot, rpanel (>= 1.1-4), tcltk
Description: This is software linked to the book
  'Applied Smoothing Techniques for Data Analysis -
  The Kernel Approach with S-Plus Illustrations' Oxford University Press.
License: GPL (>= 2)
LazyData: TRUE
Encoding: UTF-8
RoxygenNote: 7.2.1
NeedsCompilation: yes
Packaged: 2024-02-17 15:23:19 UTC; adrianbowman
Repository: CRAN
Date/Publication: 2024-02-17 17:40:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 12:05:16 UTC; unix
Archs: sm.so.dSYM
