The files flock.c and flock.h are provided for those systems which
don't have flock()

The file mhlock.c is to provide the LCK_BEL style locking available in
'mh'. This creates a lock file in the mail spool directory e.g. if the
mail directory is '/usr/spool/mail' and there is a mailbox 'asjl' in
there then a file '/usr/spool/mail/asjl.lock' will be created whilst
mail is being delivered. This provides a safe mechanism for those who
require locking over NFS mounted systems and who use this style of
locking in their mail interface programs. Define MH_LCK_BEL in the
Makefile if you want this behaviour.

Andy Linton <Andy.Linton@comp.vuw.ac.nz>
Fri Nov 23 14:18:58 NZDT 1990
