\name{dogs}

\alias{dogs}

\title{Coronary sinus potassium in dogs}

\description{
Measurements of coronary sinus potassium (mil equivalent per litre)
were made at (1,3,5,7,9,11,13) minutes after coronary occlusion in a 
number of different dogs.  There are four treatment groups 
(group 1 is the control).  The paper by Grizzle and Allen provides a 
full description of the treatments.  A few subjects develop ventricular 
fibrillation (see paper for details).

The variables are:
\tabular{ll}{
	\code{Group}                        \tab  a treatment group indicator \cr
	\code{P1, P3, P5, P7, P9, P11, P13} \tab  measurements at indicated times 
}

J.E.Grizzle & D.M.Allen (1969). Analysis of growth and dose response curves.  Biometrics vol.25, p.357-381
}

\keyword{smooth}
\keyword{regression}

